@@ -1,7 +1,7 @@
 /*
  * The parse tree transformation module for SIP.
  *
- * Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -64,8 +64,7 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
 static void resolvePySigTypes(sipSpec *,moduleDef *,classDef *,overDef *,signatureDef *,int);
 static void resolveVariableType(sipSpec *,varDef *);
 static void fatalNoDefinedType(scopedNameDef *);
-static void getBaseType(sipSpec *,moduleDef *,classDef *,argDef *);
-static void resolveType(sipSpec *,moduleDef *,classDef *,argDef *, int);
+static void resolveType(sipSpec *,moduleDef *,classDef *,argDef *,int);
 static void searchClassScope(sipSpec *,classDef *,scopedNameDef *,argDef *);
 static void searchMappedTypes(sipSpec *,moduleDef *,scopedNameDef *,argDef *);
 static void searchEnums(sipSpec *,scopedNameDef *,argDef *);
@@ -90,6 +89,7 @@ static void setStringPoolOffsets(sipSpec *pt);
 static const char *templateString(const char *src, scopedNameDef *names,
         scopedNameDef *values);
 static mappedTypeDef *copyTemplateType(mappedTypeDef *mtd, argDef *ad);
+static void checkProperties(classDef *cd);
 
 
 /*
@@ -114,7 +114,20 @@ void transform(sipSpec *pt)
 
     while (cd != NULL)
     {
-        classDef *next = cd -> next;
+        classDef *next;
+
+        /*
+         * Take the opportunity to strip any classes that are only template
+         * arguments.
+         */
+        while (isTemplateArg(cd))
+            if ((cd = cd->next) == NULL)
+                break;
+
+        if (cd == NULL)
+            break;
+
+        next = cd -> next;
 
         cd -> next = rev;
         rev = cd;
@@ -283,9 +296,12 @@ void transform(sipSpec *pt)
             cd->iff->ifacenr = cd->real->iff->ifacenr;
     }
 
-    /* Mark classes that can have an assignment helper. */
+    /* Additional class specific checks. */
     for (cd = pt->classes; cd != NULL; cd = cd->next)
+    {
         checkAssignmentHelper(pt, cd);
+        checkProperties(cd);
+    }
 
     setStringPoolOffsets(pt);
 }
@@ -641,13 +657,6 @@ static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd)
             /* Previous error checking means this will always work. */
             dcd = findAltClassImplementation(pt, al->arg.u.mtd);
 
-        /*
-         * If the destination class is in a different module then use
-         * a proxy.
-         */
-        if (dcd->iff->module != mod)
-            dcd = getProxy(mod, dcd);
-
         /* Create the new ctor. */
         ct = sipMalloc(sizeof (ctorDef));
 
@@ -665,6 +674,16 @@ static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd)
         ad->defval = NULL;
         ad->u.cd = cd;
 
+        /*
+         * If the destination class is in a different module then use
+         * a proxy.
+         */
+        if (dcd->iff->module != mod)
+        {
+            ifaceFileIsUsed(&mod->used, ad);
+            dcd = getProxy(mod, dcd);
+        }
+
         ifaceFileIsUsed(&dcd->iff->used, ad);
 
         ct->pysig.nrArgs = 1;
@@ -776,7 +795,8 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
 
         if (mdhead == NULL)
         {
-            fatal("One of the arguments of ");
+            fatal("%s:%d: One of the arguments of ", od->sloc.name,
+                    od->sloc.linenr);
             prOverloadName(stderr, od);
             fatal(" must be a class or enum\n");
         }
@@ -791,14 +811,16 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
         {
             if (second)
             {
-                fatal("The first argument of ");
+                fatal("%s:%d: The first argument of ", od->sloc.name,
+                        od->sloc.linenr);
                 prOverloadName(stderr, od);
                 fatal(" must be a class or enum\n");
             }
 
             if (mod != gmd->module && arg0->atype == enum_type)
             {
-                fatal("The first argument of ");
+                fatal("%s:%d: The first argument of ", od->sloc.name,
+                        od->sloc.linenr);
                 prOverloadName(stderr, od);
                 fatal(" must be a class\n");
             }
@@ -887,8 +909,11 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
 
         *odhead = od;
 
-        /* Remove the first argument of comparison operators. */
-        if (isRichCompareSlot(md))
+        /*
+         * Remove the first argument of inplace numeric operators and
+         * comparison operators.
+         */
+        if (isInplaceNumberSlot(md) || isRichCompareSlot(md))
         {
             /* Remember if the argument was a pointer. */
             if (arg0->nrderefs > 0)
@@ -897,6 +922,10 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
             *arg0 = *arg1;
             od->pysig.nrArgs = 1;
         }
+
+        /* Remove the only argument of unary operators. */
+        if (isZeroArgSlot(md))
+            od->pysig.nrArgs = 0;
     }
 }
 
@@ -1338,7 +1367,8 @@ static void transformTypedefs(sipSpec *pt, moduleDef *mod)
 
     for (td = pt->typedefs; td != NULL; td = td->next)
         if (td->module == mod)
-            getBaseType(pt, td->module, td->ecd, &td->type);
+            if (td->ecd == NULL || !isTemplateClass(td->ecd))
+                resolveType(pt, td->module, td->ecd, &td->type, FALSE);
 }
 
 
@@ -1377,14 +1407,23 @@ static void transformCtors(sipSpec *pt, classDef *cd)
 
         /*
          * Now check that the Python signature doesn't conflict with an
-         * earlier one.
+         * earlier one.  If there is %MethodCode then assume that it will
+         * handle any potential conflicts.
          */
-        for (prev = cd->ctors; prev != ct; prev = prev->next)
-            if (samePythonSignature(&prev->pysig, &ct->pysig))
+        if (ct->methodcode == NULL)
+        {
+            for (prev = cd->ctors; prev != ct; prev = prev->next)
             {
-                fatalScopedName(classFQCName(cd));
-                fatal(" has ctors with the same Python signature\n");
+                if (prev->methodcode != NULL)
+                    continue;
+
+                if (samePythonSignature(&prev->pysig, &ct->pysig))
+                {
+                    fatalScopedName(classFQCName(cd));
+                    fatal(" has ctors with the same Python signature\n");
+                }
             }
+        }
 
         if (isDeprecatedClass(cd))
             setIsDeprecatedCtor(ct);
@@ -1403,7 +1442,7 @@ static void transformCasts(sipSpec *pt, classDef *cd)
     {
         classDef *dcd;
 
-        getBaseType(pt, cd->iff->module, cd, &al->arg);
+        resolveType(pt, cd->iff->module, cd, &al->arg, FALSE);
 
         if (al->arg.atype == class_type)
             dcd = al->arg.u.cd;
@@ -1524,35 +1563,44 @@ static void transformScopeOverloads(sipSpec *pt, classDef *c_scope,
 
         /*
          * Now check that the Python signature doesn't conflict with an earlier
-         * one.
+         * one.  If there is %MethodCode then assume that it will handle any
+         * potential conflicts.
          */
-        for (prev = overs; prev != od; prev = prev->next)
+        if (od->methodcode == NULL)
         {
-            if (prev->common != od->common)
-                continue;
-
-            /* They can only conflict if one is unversioned. */
-            if (prev->api_range != NULL && od->api_range != NULL)
-                continue;
-
-            if (samePythonSignature(&prev->pysig, &od->pysig))
+            for (prev = overs; prev != od; prev = prev->next)
             {
-                ifaceFileDef *iff;
+                if (prev->common != od->common)
+                    continue;
 
-                if (mt_scope != NULL)
-                    iff = mt_scope->iff;
-                else if (c_scope != NULL)
-                    iff = c_scope->iff;
-                else
-                    iff = NULL;
+                if (prev->methodcode != NULL)
+                    continue;
 
-                if (iff != NULL)
+                /* They can only conflict if one is unversioned. */
+                if (prev->api_range != NULL && od->api_range != NULL)
+                    continue;
+
+                if (samePythonSignature(&prev->pysig, &od->pysig))
                 {
-                    fatalScopedName(iff->fqcname);
-                    fatal("::");
-                }
+                    ifaceFileDef *iff;
 
-                fatal("%s() has overloaded functions with the same Python signature\n", od->common->pyname->text);
+                    fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
+                    if (mt_scope != NULL)
+                        iff = mt_scope->iff;
+                    else if (c_scope != NULL)
+                        iff = c_scope->iff;
+                    else
+                        iff = NULL;
+
+                    if (iff != NULL)
+                    {
+                        fatalScopedName(iff->fqcname);
+                        fatal("::");
+                    }
+
+                    fatal("%s() has overloaded functions with the same Python signature\n", od->common->pyname->text);
+                }
             }
         }
 
@@ -1826,14 +1874,15 @@ static void resolveCtorTypes(sipSpec *pt,classDef *scope,ctorDef *ct)
     /* Handle any C++ signature. */
     if (ct->cppsig != NULL && ct->cppsig != &ct->pysig)
         for (a = 0; a < ct -> cppsig -> nrArgs; ++a)
-            getBaseType(pt, scope->iff->module, scope, &ct->cppsig->args[a]);
+            resolveType(pt, scope->iff->module, scope, &ct->cppsig->args[a],
+                    TRUE);
  
     /* Handle the Python signature. */
     for (a = 0; a < ct -> pysig.nrArgs; ++a)
     {
         argDef *ad = &ct -> pysig.args[a];
 
-        getBaseType(pt, scope->iff->module, scope, ad);
+        resolveType(pt, scope->iff->module, scope, ad, FALSE);
 
         if (!supportedType(scope,NULL,ad,FALSE) && (ct -> cppsig == &ct -> pysig || ct -> methodcode == NULL))
         {
@@ -1860,10 +1909,10 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
     {
         int a;
 
-        getBaseType(pt,mod, c_scope, &od->cppsig->result);
+        resolveType(pt, mod, c_scope, &od->cppsig->result, TRUE);
 
         for (a = 0; a < od->cppsig->nrArgs; ++a)
-            getBaseType(pt, mod, c_scope, &od->cppsig->args[a]);
+            resolveType(pt, mod, c_scope, &od->cppsig->args[a], TRUE);
     }
  
     /* Handle the Python signature. */
@@ -1875,26 +1924,29 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
     if (isSSizeReturnSlot(od->common))
         if ((res->atype != ssize_type && res->atype != int_type) || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return SIP_SSIZE_T\n",
-                    od->common->pyname->text);
+            fatal("%s:%d: %s slots must return SIP_SSIZE_T\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 
     /* These slots must return int. */
     if (isIntReturnSlot(od->common))
         if (res->atype != int_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return int\n", od->common->pyname->text);
+            fatal("%s:%d: %s slots must return int\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 
     /* These slots must return void. */
     if (isVoidReturnSlot(od->common))
         if (res->atype != void_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return void\n", od->common->pyname->text);
+            fatal("%s:%d: %s slots must return void\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 
     /* These slots must return long. */
     if (isLongReturnSlot(od->common))
         if (res->atype != long_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return long\n", od->common->pyname->text);
+            fatal("%s:%d: %s slots must return long\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 }
 
 
@@ -1911,27 +1963,31 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
     {
         if (issignal)
         {
+            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
             if (scope != NULL)
             {
                 fatalScopedName(classFQCName(scope));
                 fatal("::");
             }
 
-            fatal("%s() signals must return void\n",od -> cppname);
+            fatal("%s() signals must return void\n", od->cppname);
         }
 
-        getBaseType(pt, mod, scope, res);
+        resolveType(pt, mod, scope, res, FALSE);
 
         /* Results must be simple. */
         if (!supportedType(scope,od,res,FALSE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL))
         {
+            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
             if (scope != NULL)
             {
                 fatalScopedName(classFQCName(scope));
                 fatal("::");
             }
 
-            fatal("%s() unsupported function return type - provide %%MethodCode and a %s signature\n",od -> cppname,(pt -> genc ? "C" : "C++"));
+            fatal("%s() unsupported function return type - provide %%MethodCode and a %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
         }
     }
 
@@ -1939,7 +1995,7 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
     {
         argDef *ad = &pysig -> args[a];
 
-        getBaseType(pt, mod, scope, ad);
+        resolveType(pt, mod, scope, ad, FALSE);
 
         if (ad -> atype == slotcon_type)
             resolvePySigTypes(pt, mod, scope, od, ad->u.sa, TRUE);
@@ -1952,6 +2008,8 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
         {
             if (!supportedType(scope,od,ad,FALSE))
             {
+                fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
                 if (scope != NULL)
                 {
                     fatalScopedName(classFQCName(scope));
@@ -1963,6 +2021,9 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
         }
         else if (!supportedType(scope,od,ad,TRUE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL || (isVirtual(od) && od -> virthandler -> virtcode == NULL)))
         {
+            if (od->sloc.name != NULL)
+                fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
             if (scope != NULL)
             {
                 fatalScopedName(classFQCName(scope));
@@ -1970,9 +2031,9 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
             }
 
             if (isVirtual(od))
-                fatal("%s() unsupported function argument type - provide %%MethodCode, a valid %%VirtualCatcherCode and a valid C++ signature\n",od -> cppname);
+                fatal("%s() unsupported function argument type - provide %%MethodCode, a valid %%VirtualCatcherCode and a valid C++ signature\n", od->cppname);
 
-            fatal("%s() unsupported function argument type - provide %%MethodCode and a valid %s signature\n",od -> cppname,(pt -> genc ? "C" : "C++"));
+            fatal("%s() unsupported function argument type - provide %%MethodCode and a valid %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
         }
 
         if (scope != NULL)
@@ -1989,7 +2050,7 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     int bad = TRUE;
     argDef *vtype = &vd->type;
 
-    getBaseType(pt, vd->module, vd->ecd, vtype);
+    resolveType(pt, vd->module, vd->ecd, vtype, FALSE);
 
     switch (vtype->atype)
     {
@@ -2024,6 +2085,9 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     case enum_type:
     case bool_type:
     case cbool_type:
+    case byte_type:
+    case sbyte_type:
+    case ubyte_type:
     case ushort_type:
     case short_type:
     case uint_type:
@@ -2041,6 +2105,8 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     case pycallable_type:
     case pyslice_type:
     case pytype_type:
+    case pybuffer_type:
+    case capsule_type:
         /* These are supported without pointers or references. */
 
         if (!isReference(vtype) && vtype->nrderefs == 0)
@@ -2154,6 +2220,9 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case enum_type:
     case bool_type:
     case cbool_type:
+    case byte_type:
+    case sbyte_type:
+    case ubyte_type:
     case ushort_type:
     case short_type:
     case uint_type:
@@ -2171,6 +2240,8 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case pycallable_type:
     case pyslice_type:
     case pytype_type:
+    case pybuffer_type:
+    case capsule_type:
         if (isReference(ad))
         {
             if (isConstArg(ad))
@@ -2446,11 +2517,13 @@ int sameSignature(signatureDef *sd1,signatureDef *sd2,int strict)
 #define pyAsFloat(t)    ((t) == cfloat_type || (t) == float_type || \
             (t) == cdouble_type || (t) == double_type)
 #define pyAsInt(t)  ((t) == bool_type || (t) == ssize_type || \
+            (t) == byte_type || (t) == sbyte_type || (t) == ubyte_type || \
             (t) == short_type || (t) == ushort_type || \
             (t) == cint_type || (t) == int_type || (t) == uint_type)
 #define pyAsLong(t) ((t) == long_type || (t) == longlong_type)
 #define pyAsULong(t)    ((t) == ulong_type || (t) == ulonglong_type)
 #define pyAsAuto(t) ((t) == bool_type || \
+            (t) == byte_type || (t) == sbyte_type || (t) == ubyte_type || \
             (t) == short_type || (t) == ushort_type || \
             (t) == int_type || (t) == uint_type || \
             (t) == float_type || (t) == double_type)
@@ -2539,6 +2612,12 @@ int sameBaseType(argDef *a1, argDef *a2)
         if (a1->atype == defined_type && a2->atype == mapped_type)
             return compareScopedNames(a1->u.snd, a2->u.mtd->iff->fqcname) == 0;
 
+        if (a1->atype == enum_type && a2->atype == defined_type)
+            return compareScopedNames(a1->u.ed->fqcname, a2->u.snd) == 0;
+
+        if (a1->atype == defined_type && a2->atype == enum_type)
+            return compareScopedNames(a1->u.snd, a2->u.ed->fqcname) == 0;
+
         return FALSE;
     }
 
@@ -2576,9 +2655,17 @@ int sameBaseType(argDef *a1, argDef *a2)
                 return FALSE;
 
             for (a = 0; a < td1->types.nrArgs; ++a)
-                if (!sameBaseType(&td1->types.args[a], &td2->types.args[a]))
+            {
+                argDef *td1ad = &td1->types.args[a];
+                argDef *td2ad = &td2->types.args[a];
+
+                if (td1ad->nrderefs != td2ad->nrderefs)
                     return FALSE;
 
+                if (!sameBaseType(td1ad, td2ad))
+                    return FALSE;
+            }
+
             break;
         }
 
@@ -2628,7 +2715,6 @@ static int samePythonSignature(signatureDef *sd1, signatureDef *sd2)
     }
 
     return (a1 < 0 && a2 < 0);
-
 }
 
 
@@ -2783,16 +2869,6 @@ static void scopeDefaultValue(sipSpec *pt,classDef *cd,argDef *ad)
 }
 
 
-/*
- * Make sure a type is a base type.
- */
-static void getBaseType(sipSpec *pt, moduleDef *mod, classDef *c_scope,
-        argDef *type)
-{
-    resolveType(pt, mod, c_scope, type, FALSE);
-}
-
-
 /*
  * Resolve a type if possible.
  */
@@ -2839,7 +2915,7 @@ static void resolveType(sipSpec *pt, moduleDef *mod, classDef *c_scope,
         int sa;
 
         for (sa = 0; sa < type->u.sa->nrArgs; ++sa)
-            getBaseType(pt, mod, c_scope, &type->u.sa->args[sa]);
+            resolveType(pt, mod, c_scope, &type->u.sa->args[sa], FALSE);
     }
 
     /* See if the type refers to an instantiated template. */
@@ -2924,11 +3000,17 @@ static mappedTypeDef *instantiateMappedTypeTemplate(sipSpec *pt, moduleDef *mod,
             mappedtype_iface, NULL, type);
     mtd->iff->module = mod;
 
+    mtd->mtflags = mtt->mt->mtflags;
     mtd->doctype = templateString(mtt->mt->doctype, type_names, type_values);
 
-    appendCodeBlock(&mtd->iff->hdrcode, templateCode(pt, &mtd->iff->used, mtt->mt->iff->hdrcode, type_names, type_values));
-    mtd->convfromcode = templateCode(pt, &mtd->iff->used, mtt->mt->convfromcode, type_names, type_values);
-    mtd->convtocode = templateCode(pt, &mtd->iff->used, mtt->mt->convtocode, type_names, type_values);
+    appendCodeBlockList(&mtd->iff->hdrcode,
+            templateCode(pt, &mtd->iff->used, mtt->mt->iff->hdrcode,
+                    type_names, type_values));
+
+    mtd->convfromcode = templateCode(pt, &mtd->iff->used,
+            mtt->mt->convfromcode, type_names, type_values);
+    mtd->convtocode = templateCode(pt, &mtd->iff->used, mtt->mt->convtocode,
+            type_names, type_values);
 
     mtd->next = pt->mappedtypes;
     pt->mappedtypes = mtd;
@@ -3517,3 +3599,38 @@ static int generatingCodeForModule(sipSpec *pt, moduleDef *mod)
 
     return (pt->module == mod);
 }
+
+
+/*
+ * Check that any properties are valid.
+ */
+static void checkProperties(classDef *cd)
+{
+    propertyDef *pd;
+
+    for (pd = cd->properties; pd != NULL; pd = pd->next)
+    {
+        if (findMethod(cd, pd->get) == NULL)
+            fatal("Property %s.%s has no get method %s()\n", cd->pyname->text,
+                    pd->name->text, pd->get);
+
+        if (pd->set != NULL && findMethod(cd, pd->set) == NULL)
+            fatal("Property %s.%s has no set method %s()\n", cd->pyname->text,
+                    pd->name->text, pd->set);
+    }
+}
+
+
+/*
+ * Return the method of a class with a given name.
+ */
+memberDef *findMethod(classDef *cd, const char *name)
+{
+    memberDef *md;
+
+    for (md = cd->members; md != NULL; md = md->next)
+        if (strcmp(md->pyname->text, name) == 0)
+            break;
+
+    return md;
+}
