@@ -2,7 +2,7 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 #
 # This file is part of SIP.
 #
@@ -202,7 +202,7 @@ class Makefile:
     def __init__(self, configuration, console=0, qt=0, opengl=0, python=0,
                  threaded=0, warnings=1, debug=0, dir=None,
                  makefile="Makefile", installs=None, universal=None,
-                 arch=None):
+                 arch=None, deployment_target=None):
         """Initialise an instance of the target.  All the macros are left
         unchanged allowing scripts to manipulate them at will.
 
@@ -228,6 +228,8 @@ class Makefile:
         binary.  If it is None then the value is taken from the configuration.
         arch is the space separated MacOS/X architectures to build.  If it is
         None then it is taken from the configuration.
+        deployment_target MacOS/X deployment target.  If it is None then it is
+        taken from the configuration.
         """
         if qt:
             if not hasattr(configuration, "qt_version"):
@@ -254,6 +256,7 @@ class Makefile:
         self._debug = debug
         self._makefile = makefile
         self._installs = installs
+        self._infix = ""
 
         # Make sure the destination directory is an absolute path.
         if dir:
@@ -274,6 +277,11 @@ class Makefile:
         else:
             self._arch = arch
 
+        if deployment_target is None:
+            self._deployment_target = configuration.deployment_target
+        else:
+            self._deployment_target = deployment_target
+
         self._finalised = 0
 
         # Copy the macros and convert them all to instance lists.
@@ -346,12 +354,16 @@ class Makefile:
             win_exceptions = ("exceptions" in wcfg)
             win_rtti = ("rtti" in wcfg)
             win_stl = ("stl" in wcfg)
+
+            qt_version = self.config.qt_version
         else:
             win_shared = 1
             win_exceptions = 0
             win_rtti = 0
             win_stl = 0
 
+            qt_version = 0
+
         # Get what we are going to transform.
         cflags = _UniqueList()
         cflags.extend(self.extra_cflags)
@@ -411,10 +423,18 @@ class Makefile:
         rpaths = _UniqueList()
 
         for l in self.extra_lib_dirs:
+            l_dir = os.path.dirname(l)
+
+            # This is a hack to ignore PyQt's internal support libraries.
+            if '/qpy/' in l_dir:
+                continue
+
             # Ignore relative directories.  This is really a hack to handle
             # SIP v3 inter-module linking.
-            if os.path.dirname(l) not in ("", ".", ".."):
-                rpaths.append(l)
+            if l_dir in ("", ".", ".."):
+                continue
+
+            rpaths.append(l)
 
         if self._python:
             incdir.append(self.config.py_inc_dir)
@@ -532,13 +552,21 @@ class Makefile:
             lflags.extend(self.optional_list("LFLAGS_THREAD"))
 
         if self._qt:
+            # Get the name of the mkspecs directory.
+            try:
+                specd_base = self.config.qt_data_dir
+            except AttributeError:
+                specd_base = self.config.qt_dir
+
+            mkspecs = os.path.join(specd_base, "mkspecs")
+
             if self.generator != "UNIX" and win_shared:
                 defines.append("QT_DLL")
 
             if not self._debug:
                 defines.append("QT_NO_DEBUG")
 
-            if self.config.qt_version >= 0x040000:
+            if qt_version >= 0x040000:
                 for mod in self._qt:
                     # Note that qmake doesn't define anything for QtHelp.
                     if mod == "QtCore":
@@ -569,6 +597,9 @@ class Makefile:
                         defines.append("QT_XMLPATTERNS_LIB")
                     elif mod == "phonon":
                         defines.append("QT_PHONON_LIB")
+
+                    if qt_version >= 0x050000 and mod in ("QtSql", "QtTest"):
+                        defines.append("QT_WIDGETS_LIB")
             elif self._threaded:
                 defines.append("QT_THREAD_SUPPORT")
 
@@ -577,7 +608,11 @@ class Makefile:
             libdir.extend(libdir_qt)
             rpaths.extend(libdir_qt)
 
-            if self.config.qt_version >= 0x040000:
+            if qt_version >= 0x040000:
+                # Try and read QT_LIBINFIX from qconfig.pri.
+                qconfig = os.path.join(mkspecs, "qconfig.pri")
+                self._infix = self._extract_value(qconfig, "QT_LIBINFIX")
+
                 # For Windows: the macros that define the dependencies on
                 # Windows libraries.
                 wdepmap = {
@@ -622,7 +657,7 @@ class Makefile:
                 # include QtGui and QtNetwork as a dependency any longer.  This
                 # seems to be a bug in Qt v4.2.0.  We explicitly set the
                 # dependencies here.
-                if self.config.qt_version >= 0x040200 and "QtAssistant" in self._qt:
+                if qt_version >= 0x040200 and "QtAssistant" in self._qt:
                     if "QtGui" not in self._qt:
                         self._qt.append("QtGui")
                     if "QtNetwork" not in self._qt:
@@ -660,7 +695,7 @@ class Makefile:
                 qt_lib = self.config.qt_lib
 
                 if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE") and win_shared:
-                    qt_lib = qt_lib + version_to_string(self.config.qt_version).replace(".", "")
+                    qt_lib = qt_lib + version_to_string(qt_version).replace(".", "")
 
                     if self.config.qt_edition == "non-commercial":
                         qt_lib = qt_lib + "nc"
@@ -669,32 +704,54 @@ class Makefile:
                 libs.extend(self._dependent_libs(self.config.qt_lib))
 
             # Handle header directories.
-            try:
-                specd_base = self.config.qt_data_dir
-            except AttributeError:
-                specd_base = self.config.qt_dir
-
-            specd = os.path.join(specd_base, "mkspecs", "default")
+            specd = os.path.join(mkspecs, "default")
 
             if not os.access(specd, os.F_OK):
-                specd = os.path.join(specd_base, "mkspecs", self.config.platform)
+                specd = os.path.join(mkspecs, self.config.platform)
 
             incdir.append(specd)
 
             qtincdir = self.optional_list("INCDIR_QT")
 
             if qtincdir:
-                if self.config.qt_version >= 0x040000:
+                if qt_version >= 0x040000:
                     for mod in self._qt:
                         if mod == "QAxContainer":
                             incdir.append(os.path.join(qtincdir[0], "ActiveQt"))
                         elif self._is_framework(mod):
-                            if mod == "QtAssistant" and self.config.qt_version < 0x040202:
+                            idir = libdir_qt[0]
+
+                            if mod == "QtAssistant" and qt_version < 0x040202:
                                 mod = "QtAssistantClient"
 
-                            incdir.append(os.path.join(libdir_qt[0], mod + ".framework", "Headers"))
+                            incdir.append(os.path.join(idir,
+                                    mod + ".framework", "Headers"))
+
+                            if qt_version >= 0x050000:
+                                if mod == "QtGui":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWidgets.framework", "Headers"))
+                                    incdir.append(os.path.join(idir,
+                                            "QtPrintSupport.framework",
+                                            "Headers"))
+                                elif mod == "QtWebKit":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWebKitWidgets.framework",
+                                            "Headers"))
                         else:
-                            incdir.append(os.path.join(qtincdir[0], mod))
+                            idir = qtincdir[0]
+
+                            incdir.append(os.path.join(idir, mod))
+
+                            if qt_version >= 0x050000:
+                                if mod == "QtGui":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWidgets"))
+                                    incdir.append(os.path.join(idir,
+                                            "QtPrintSupport"))
+                                elif mod == "QtWebKit":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWebKitWidgets"))
 
                 # This must go after the module include directories.
                 incdir.extend(qtincdir)
@@ -706,7 +763,7 @@ class Makefile:
             libs.extend(self.optional_list("LIBS_OPENGL"))
 
         if self._qt or self._opengl:
-            if self.config.qt_version < 0x040000 or self._opengl or "QtGui" in self._qt:
+            if qt_version < 0x040000 or self._opengl or "QtGui" in self._qt:
                 incdir.extend(self.optional_list("INCDIR_X11"))
                 libdir.extend(self.optional_list("LIBDIR_X11"))
                 libs.extend(self.optional_list("LIBS_X11"))
@@ -760,13 +817,20 @@ class Makefile:
                 lib = mname
             else:
                 lib = "QtAssistantClient"
+        elif mname == "QtWebKit" and self.config.qt_version >= 0x050000:
+            lib = "QtWebKitWidgets"
         else:
             lib = mname
 
+        lib += self._infix
+
         if self._debug:
             if sys.platform == "win32":
                 lib = lib + "d"
-            elif self.config.qt_version < 0x040200 or sys.platform == "darwin":
+            elif sys.platform == "darwin":
+                if not self._is_framework(mname):
+                    lib = lib + "_debug"
+            elif self.config.qt_version < 0x040200:
                 lib = lib + "_debug"
 
         if sys.platform == "win32" and "shared" in self.config.qt_winconfig.split():
@@ -852,8 +916,6 @@ class Makefile:
         clib is the library name in cannonical form.
         framework is set of the library is implemented as a MacOS framework.
         """
-        prl_libs = []
-
         if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".prl")
         elif sys.platform == "darwin" and framework:
@@ -861,27 +923,42 @@ class Makefile:
         else:
             prl_name = os.path.join(self.config.qt_lib_dir, "lib" + clib + ".prl")
 
-        if os.access(prl_name, os.F_OK):
+        libs = self._extract_value(prl_name, "QMAKE_PRL_LIBS").split()
+
+        if self.config.qt_version >= 0x050000 and clib == "QtGui":
+            for xtra in ("QtWidgets", "QtPrintSupport"):
+                libs.extend(self.platform_lib(xtra, framework).split())
+
+        return libs
+
+    def _extract_value(self, fname, vname):
+        """Return the stripped value from a name=value line in a file.
+
+        fname is the name of the file.
+        vname is the name of the value.
+        """
+        value = ""
+
+        if os.access(fname, os.F_OK):
             try:
-                f = open(prl_name, "r")
+                f = open(fname, "r")
             except IOError:
-                error("Unable to open \"%s\"" % prl_name)
+                error("Unable to open \"%s\"" % fname)
 
             line = f.readline()
             while line:
                 line = line.strip()
                 if line and line[0] != "#":
                     eq = line.find("=")
-                    if eq > 0 and line[:eq].strip() == "QMAKE_PRL_LIBS":
-                        prl_libs = line[eq + 1:].split()
+                    if eq > 0 and line[:eq].strip() == vname:
+                        value = line[eq + 1:].strip()
                         break
 
                 line = f.readline()
 
             f.close()
 
-        return prl_libs
-
+        return value
 
     def parse_build_file(self, filename):
         """
@@ -1023,6 +1100,9 @@ class Makefile:
 
         mfile is the file object.
         """
+        if self._deployment_target:
+            mfile.write("export MACOSX_DEPLOYMENT_TARGET = %s\n" % self._deployment_target)
+
         mfile.write("CC = %s\n" % self.required_string("CC"))
         mfile.write("CXX = %s\n" % self.required_string("CXX"))
         mfile.write("LINK = %s\n" % self.required_string("LINK"))
@@ -1331,7 +1411,8 @@ class ModuleMakefile(Makefile):
     def __init__(self, configuration, build_file, install_dir=None, static=0,
                  console=0, qt=0, opengl=0, threaded=0, warnings=1, debug=0,
                  dir=None, makefile="Makefile", installs=None, strip=1,
-                 export_all=0, universal=None, arch=None):
+                 export_all=0, universal=None, arch=None,
+                 deployment_target=None):
         """Initialise an instance of a module Makefile.
 
         build_file is the file containing the target specific information.  If
@@ -1345,7 +1426,7 @@ class ModuleMakefile(Makefile):
         increases the size of the module and slows down module load times but
         may avoid problems with modules that use exceptions.  The default is 0.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs, universal, arch)
+        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs, universal, arch, deployment_target)
 
         self._build = self.parse_build_file(build_file)
         self._install_dir = install_dir
@@ -1647,7 +1728,8 @@ class SIPModuleMakefile(ModuleMakefile):
     def __init__(self, configuration, build_file, install_dir=None, static=0,
                  console=0, qt=0, opengl=0, threaded=0, warnings=1, debug=0,
                  dir=None, makefile="Makefile", installs=None, strip=1,
-                 export_all=0, universal=None, arch=None, prot_is_public=0):
+                 export_all=0, universal=None, arch=None, prot_is_public=0,
+                 deployment_target=None):
         """Initialise an instance of a SIP generated module Makefile.
 
         prot_is_public is set if "protected" is to be redefined as "public".
@@ -1658,7 +1740,8 @@ class SIPModuleMakefile(ModuleMakefile):
         """
         ModuleMakefile.__init__(self, configuration, build_file, install_dir,
                 static, console, qt, opengl, threaded, warnings, debug, dir,
-                makefile, installs, strip, export_all, universal, arch)
+                makefile, installs, strip, export_all, universal, arch,
+                deployment_target)
 
         self._prot_is_public = prot_is_public
 
@@ -1680,14 +1763,14 @@ class ProgramMakefile(Makefile):
     def __init__(self, configuration, build_file=None, install_dir=None,
                  console=0, qt=0, opengl=0, python=0, threaded=0, warnings=1,
                  debug=0, dir=None, makefile="Makefile", installs=None,
-                 universal=None, arch=None):
+                 universal=None, arch=None, deployment_target=None):
         """Initialise an instance of a program Makefile.
 
         build_file is the file containing the target specific information.  If
         it is a dictionary instead then its contents are validated.
         install_dir is the directory the target will be installed in.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs, universal, arch)
+        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs, universal, arch, deployment_target)
 
         self._install_dir = install_dir
 
@@ -1731,6 +1814,9 @@ class ProgramMakefile(Makefile):
 
         build.extend(self.optional_list("CXXFLAGS"))
 
+        # This is for Qt5.
+        build.extend(self.optional_list("CXXFLAGS_APP"))
+
         # Borland requires all flags to precede all file names.
         if self.generator != "BMAKE":
             build.append(source)
@@ -2080,6 +2166,9 @@ def create_config_module(module, template, content, macros=None):
 
         line = sf.readline()
 
+    df.close()
+    sf.close()
+
 
 def version_to_sip_tag(version, tags, description):
     """Convert a version number to a SIP tag.
@@ -2241,10 +2330,13 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 self._openfile(nextfile)
                 return self.readline()
 
-            if not line and self.filestack:
-                self.currentfile = self.filestack.pop()
-                self.path = self.pathstack.pop()
-                return self.readline()
+            if not line:
+                self.currentfile.close()
+
+                if self.filestack:
+                    self.currentfile = self.filestack.pop()
+                    self.path = self.pathstack.pop()
+                    return self.readline()
 
             return line
 
@@ -2327,7 +2419,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                     if orig_rhs is not None:
                         rhs = orig_rhs + " " + rhs
 
-                raw[lhs] = rhs
+                raw[lhs] = _expand_macro_value(raw, rhs, properties)
 
         line = f.readline()
 
@@ -2343,7 +2435,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
     for lhs in list(raw.keys()):
         # Strip any prefix.
-        if lhs.find(macro_prefix) == 0:
+        if lhs.startswith(macro_prefix):
             reflhs = lhs[len(macro_prefix):]
         else:
             reflhs = lhs
@@ -2354,48 +2446,6 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
         rhs = raw[lhs]
 
-        # Resolve any references.
-        estart = rhs.find("$$(")
-        mstart = rhs.find("$$")
-
-        while mstart >= 0 and mstart != estart:
-            rstart = mstart + 2
-            if rstart < len(rhs) and rhs[rstart] == "{":
-                rstart = rstart + 1
-                term = "}"
-            elif rstart < len(rhs) and rhs[rstart] == "[":
-                rstart = rstart + 1
-                term = "]"
-            else:
-                term = string.whitespace
-
-            mend = rstart
-            while mend < len(rhs) and rhs[mend] not in term:
-                mend = mend + 1
-
-            lhs = rhs[rstart:mend]
-
-            if term in "}]":
-                mend = mend + 1
-
-            if term == "]":
-                if properties is None or lhs not in list(properties.keys()):
-                    error("%s: property '%s' is not defined." % (filename, lhs))
-
-                value = properties[lhs]
-            else:
-                try:
-                    value = raw[lhs]
-                except KeyError:
-                    # We used to treat this as an error, but Qt v4.3.0 has at
-                    # least one case that refers to an undefined macro.  If
-                    # qmake handles it then this must be the correct behaviour.
-                    value = ""
-
-            rhs = rhs[:mstart] + value + rhs[mend:]
-            estart = rhs.find("$$(")
-            mstart = rhs.find("$$")
-
         # Expand any POSIX style environment variables.
         pleadin = ["$$(", "$("]
 
@@ -2476,6 +2526,50 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
     return refined
 
 
+def _expand_macro_value(macros, rhs, properties):
+    """Expand the value of a macro based on ones seen so far."""
+    estart = rhs.find("$$(")
+    mstart = rhs.find("$$")
+
+    while mstart >= 0 and mstart != estart:
+        rstart = mstart + 2
+        if rstart < len(rhs) and rhs[rstart] == "{":
+            rstart = rstart + 1
+            term = "}"
+        elif rstart < len(rhs) and rhs[rstart] == "[":
+            rstart = rstart + 1
+            term = "]"
+        else:
+            term = string.whitespace
+
+        mend = rstart
+        while mend < len(rhs) and rhs[mend] not in term:
+            mend = mend + 1
+
+        lhs = rhs[rstart:mend]
+
+        if term in "}]":
+            mend = mend + 1
+
+        if term == "]":
+            # Assume a missing property expands to an empty string.
+            if properties is None:
+                value = ""
+            else:
+                value = properties.get(lhs, "")
+        else:
+            # We used to treat a missing value as an error, but Qt v4.3.0 has
+            # at least one case that refers to an undefined macro.  If qmake
+            # handles it then this must be the correct behaviour.
+            value = macros.get(lhs, "")
+
+        rhs = rhs[:mstart] + value + rhs[mend:]
+        estart = rhs.find("$$(")
+        mstart = rhs.find("$$")
+
+    return rhs
+
+
 def create_wrapper(script, wrapper, gui=0, use_arch=''):
     """Create a platform dependent executable wrapper around a Python script.
 
